@@ -26,9 +26,9 @@ Crypt::Perl::ECDSA - Elliptic curve cryptography in pure Perl
             b => ..., #isa Crypt::Perl::BigInt
             n => ..., #isa Crypt::Perl::BigInt
 
-            #Supposedly this can be deduced from the above, but I don’t
-            #see the math for this around. It’s not in libtomcryt, AFAICT.
-            #It may have to do with Schoof’s Algorithm?
+            # Supposedly this can be deduced from the above, but I don’t
+            # see the math for this around. It’s not in libtomcryt, AFAICT.
+            # It may have to do with Schoof’s Algorithm?
             h => ..., #isa Crypt::Perl::BigInt
 
             gx => ..., #isa Crypt::Perl::BigInt
@@ -40,13 +40,24 @@ Crypt::Perl::ECDSA - Elliptic curve cryptography in pure Perl
 
     my $msg = 'My message';
 
-    my $hash = Digest::SHA::sha256($msg);
+    # Deterministic signatures. This is probably the way to go
+    # for normal use cases. You can use sha1, sha224, sha256, sha384,
+    # and sha512.
+    my $det_sig = $private->sign_sha256($msg);
 
-    my $sig = $private->sign($hash);
+    my $msg_hash = Digest::SHA::sha256($msg);
 
-    die 'Wut' if !$private->verify($hash, $sig);
+    # NB: This verifies a *digest*, not the original message.
+    die 'Wut' if !$public->verify(msg_hash, $sig);
+    die 'Wut' if !$private->verify(msg_hash, $sig);
 
-    die 'Wut' if !$public->verify($hash, $sig);
+    # Signature in JSON Web Algorithm format (deterministic):
+    my $jwa_sig = $private->sign_jwa($msg);
+
+    # You can also create non-deterministic signatures. These risk a
+    # security compromise if there is any flaw in the underlying CSPRNG:
+    # Note that this signs a *digest*, not the message itself.
+    my $sig = $private->sign(msg_hash);
 
     #----------------------------------------------------------------------
 
@@ -75,7 +86,7 @@ can do.
 The security advantages of elliptic-curve cryptography (ECC) are a matter of
 some controversy. While the math itself is apparently bulletproof, there are
 varying opinions about the integrity of the various curves that are recommended
-for ECC. Some believe that some curves contain “backdoors” that would allow
+for ECC. Some believe that some curves contain backdoors that would allow
 L<NIST|https://www.nist.gov> to sniff a transmission. For more information,
 look at L<http://safecurves.cr.yp.to>.
 
@@ -86,9 +97,16 @@ C<Crypt::Perl> “has no opinion” regarding which curves you use; it ships all
 of the prime-field curves that (L<OpenSSL|http://openssl.org>) includes and
 works with any of them. You can try out custom curves as well.
 
+=head2 Deterministic Signatures
+
+This library can now create deterministic signatures, as per
+L<RFC 6979|https://tools.ietf.org/html/rfc6979>. Read that RFC to learn
+why this is a good idea. (tl;dr: It probably is, unless your special use
+case dictates otherwise.)
+
 =head1 FORMATS SUPPORTED
 
-Elliptic-curve keys can come in a variety of formats. This library supports
+Elliptic-curve keys can be in a variety of formats. This library supports
 almost all of them:
 
 =over
@@ -97,7 +115,8 @@ almost all of them:
 about explicit curve parameters.)
 
 =item Parse and export of curve points in compressed or uncompressed form,
-and parse of points in hybrid form. (L<RFC 5480|https://www.rfc-editor.org/rfc/rfc5480.txt>
+and parse of points in hybrid form.
+(NB: L<RFC 5480|https://www.rfc-editor.org/rfc/rfc5480.txt>
 prohibits use of the hybrid form.)
 
 =back
@@ -105,7 +124,7 @@ prohibits use of the hybrid form.)
 Explicit curves (i.e., giving the curve by full parameters rather than by
 name reference) may be a known curve or an arbitrary curve.
 Explicit curves may include or omit the seed value. It is omitted in output
-by default. They may also include or
+by default. Explicit curves may also include or
 omit the cofactor, but if the curve is unknown the cofactor is required.
 This is because this library’s export of explicit curves always includes the
 cofactor. While it’s not required for ECDSA, it’s recommended, and it’s
@@ -137,7 +156,8 @@ other curve parameters.
 
 =head1 ACKNOWLEDGEMENTS
 
-The ECDSA logic here is ported from Kenji Urushima’s L<jsrsasign|http://kjur.github.io/jsrsasign/>.
+Most of the ECDSA logic here is ported from Kenji Urushima’s
+L<jsrsasign|http://kjur.github.io/jsrsasign/>.
 
 Curve data is copied from OpenSSL. (See the script included in the
 distribution.)
