@@ -8,9 +8,9 @@ use Test::More;
 use Call::Context ();
 use File::Spec ();
 use File::Temp ();
+use File::Slurp ();
 use File::Which ();
 use IPC::Open3 ();
-use Module::Load ();
 
 use lib '../lib';
 
@@ -26,7 +26,7 @@ use constant BLACKLIST_CURVES => (
 
 sub openssl_version {
     my $bin = openssl_bin();
-    return scalar qx<$bin version -v -o -f>;
+    return $bin && scalar qx<$bin version -v -o -f>;
 }
 
 sub can_load_private_pem {
@@ -53,11 +53,9 @@ sub can_ecdsa {
         my $bin = openssl_bin();
 
         if ($bin) {
-            my $pid = IPC::Open3::open3( my $wtr, my $rdr, undef, "$bin ecparam -list_curves" );
-            close $wtr;
+            my $pid = open my $rdr, '-|', "$bin ecparam -list_curves";
             my $out = do { local $/; <$rdr> };
             close $rdr;
-            waitpid $pid, 0;
 
             $_ecdsa_test_err = $?;
 
@@ -73,6 +71,59 @@ sub can_ecdsa {
     return !$_ecdsa_test_err;
 }
 
+my $_can_ed25519;
+sub can_ed25519 {
+    my ($self) = @_;
+
+    if (!defined $_can_ed25519) {
+        my $bin = openssl_bin();
+
+        diag "Checking $bin for ed25519 support …";
+
+        my $cmd = "$bin genpkey -algorithm ed25519";
+
+        my $out = `$cmd`;
+
+        # On some OpenSSLs/OSes $? isn’t populated, even in failure.
+        if ($? || $out !~ m<BEGIN>) {
+            $_can_ed25519 = 0;
+            diag "$bin does not support ed25519.";
+        }
+        else {
+            $_can_ed25519 = 1;
+            diag "$bin supports ed25519.";
+        }
+    }
+
+    return $_can_ed25519;
+}
+
+my $_has_ecdsa_verify_private_bug;
+
+# Certain old OpenSSL versions (0.9.8zg … maybe others?) fail to recognize
+# the ECDSA private key in dgst’s verification logic.
+#
+# The error is:
+#
+#   0606C06E:digital envelope routines:EVP_VerifyFinal:wrong public key type:/SourceCache/OpenSSL098/OpenSSL098-52.40.1/src/crypto/evp/p_verify.c:85
+#
+sub has_ecdsa_verify_private_bug {
+    if (!defined $_has_ecdsa_verify_private_bug) {
+        die "OpenSSL can’t even do ECDSA!" if !can_ecdsa();
+
+        my $t_dir = __FILE__;
+        $t_dir =~ s<[^/]+\z><..>;
+
+        my $key_pem = File::Slurp::read_file("$t_dir/assets/prime256v1.key");
+        my $msg = "hello";
+        my $sig = pack 'H*', '30460221009aae8b7ae764d93fb97ddbab4f5ea0169175e4690e3cfd6297ccbfa3555b3c420221008519ce1ca5f944b7e42d3dd1c3ef2323dbb2404dc8ab36340077642439f3ac1c';
+
+        $_has_ecdsa_verify_private_bug = !verify_private($key_pem, $msg, 'sha1', $sig);
+    }
+
+    return $_has_ecdsa_verify_private_bug;
+}
+
 sub verify_private {
     my ($key_pem, $message, $digest_alg, $signature) = @_;
 
@@ -132,7 +183,7 @@ sub curve_oid {
 sub curve_data {
     my ($name) = @_;
 
-    Module::Load::load('Crypt::Perl::ECDSA::ECParameters');
+    require Crypt::Perl::ECDSA::ECParameters;
 
     my ($asn1, $out) = __ecparam( $name, 'explicit', Crypt::Perl::ECDSA::ECParameters::ASN1_ECParameters() );
 
@@ -142,7 +193,7 @@ sub curve_data {
 sub __ecparam {
     my ($name, $param_enc, $asn1_template) = @_;
 
-    Module::Load::load('Crypt::Perl::ASN1');
+    require Crypt::Perl::ASN1;
 
     my $bin = openssl_bin();
     my $out = qx<$bin ecparam -name $name -param_enc $param_enc -outform DER>;
@@ -155,7 +206,19 @@ sub __ecparam {
 
 my $ossl_bin;
 sub openssl_bin {
-    return $ossl_bin ||= File::Which::which('openssl');
+    return $ossl_bin ||= do {
+        diag "Looking for OpenSSL binary …";
+
+        my $bin = File::Which::which('openssl');
+
+        diag "Found OpenSSL: $bin";
+
+        $bin;
+    };
+}
+
+BEGIN {
+    diag( openssl_version() );
 }
 
 1;
